# Nut
# by Eresse <eresse@eresse.net>

# External Includes
require 'uri'
require 'cgi'
require 'typor'
require 'rxio'

# Internal Includes
require 'nut/version'
require 'nut/codes'
require 'nut/type_symbols'

# Nut Module
module Nut

	# Response Module
	module Response

		# Build Response:
		# Constructs a complete Response from a preliminary Response Hash (code, body).
		# @param [Hash] response Preliminary Response Information
		def self.build_response response

			# Set Response Code
			response[:code] ||= 200

			# Ensure Body is present
			response[:body] = '0' unless response[:body].nstr

			# Determine Content-Type
			response[:type] = TYPE_SYMBOLS[response[:type]] if response[:type].is_a? Symbol
			response[:type] ||= Typor.data(response[:body]) || 'text/html'

			# Prepare Headers
			response[:headers].merge! 'Server' => "#{SERVER_NAME} #{VERSION}", 'Content-Type' => response[:type], 'Content-Length' => response[:body].bytesize

			# Handle Redirects
			if response[:redirect]

				# Allow '301 Moved' permanent redirects (default to '302 Found' Redirects)
				response[:code] = response[:redirect].is_a?(String) ? 302 : (response[:redirect][:permanent] ? 301 : 302)

				# Set Target
				response[:headers]['Location'] = response[:redirect].is_a?(String) ? response[:redirect] : response[:redirect][:to]
			end

			# Compile Response
			compile_response response
		end

		# Compile Response:
		# Sends out an HTTP Response to the Client.
		# @param [Hash] response Response Hash
		def self.compile_response response
			"HTTP/1.1 #{response[:code]} #{CODES[response[:code]]}\r\n" +
				response[:headers].collect { |k, v| "#{k}: #{v}\r\n" }.join +
				"\r\n" + response[:body]
		end
	end
end
